/**<pre>
 * Title: iSIM
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package controller;


import gui.*;
import gui.tray.*;
import util.*;
import agent.*;
import agent.autonomous.*;
import transport.*;
import transport.protocol.*;
import map.*;

import javax.swing.JFrame;
import java.awt.*;
import java.util.*;



/* ====================================================================== */
/** This class is the core of the simulator.  It initializes the interface
 *  components, holds the agents, and controls the sockets for sending and
 *  recieving data.
 *
 */
public class Controller implements SocketReceiverListener{


	/** The chart is the visual map. */
	private ChartComp chart;

	/** The main frame which displays the map ChartComp. */
  	private SimulatorFrame frame;

	/** An interface component for viewing agent stats, etc. */
	private TrayDialog tray;

	/** The agent that is being moved, or has center of screen. */
	private Agent current_agent;

	/** The list of all access points in the simulation. */
	private Vector points = new Vector();

	/** The list of all users in the simulation. */
	private Vector users = new Vector();

	/** Components that want to listen for when information changes in controller. */
	private Vector listeners = new Vector();




    /* ===================================================================== */
    /** This constructor initializes the controller by creating the interface
     *  components, so that the user can begin interacting with the simulator.
     *	It also starts the SocketReceiver for listening to incoming messages.
     *
     */
	public Controller(){

		frame = new SimulatorFrame(this);
		//tray = new TrayDialog(this, frame);

		try{ Thread.sleep(500); }catch(Exception e){}

		frame.setVisible(true);

		SocketReceiver sr = new SocketReceiver(this, 2323);
		sr.start();

		new ControllerThread(this);

	} //Constructor




    /* ===================================================================== */
    /** This method is for adding a new user into the simulator. The user is
     *  created based on the given information, and is added into the map, and
     *  all controller listeners are notified of the new addition.
     *
     *  @param  r   The range for the new user.
     *  @param  p   The port for the new user.
     *
     */
	public void addUser(int r, int p){

		Point pt = frame.getChartPane().getScrollPosition();
		Dimension d= frame.getChartPane().getViewportSize();

		User u = new User( new Point((int)(pt.getX()+(d.width)/2)+75,(int)(pt.getY()+(d.height)/2)+50), p );

		current_agent = u;
		users.add( current_agent);

		current_agent.setRange(r);
		current_agent.setFloor(chart.getFloor().getNumber());
		current_agent.setName( "user"+ (users.indexOf(current_agent))%8 );
		((User)current_agent).setUsername( "user"+users.indexOf(current_agent) );


		int index = users.indexOf(u);


		updateAgentPosition( current_agent );

		try{
			fireControllerChange();
		}catch(Exception e){}

	} //method: addUser



    /* ===================================================================== */
    /** This method is for adding a new user into the simulator. The user is
     *  created based on the given information, and is added into the map, and
     *  all controller listeners are notified of the new addition.
     *
     *  @param  r   The range for the new user.
     *  @param  p   The port for the new user.
     *
     */
	public void addAutonomous(int r, int p, String type){

		Point pt = frame.getChartPane().getScrollPosition();
		Dimension d= frame.getChartPane().getViewportSize();

		User u;

		if( type.equalsIgnoreCase( "Searcher" ) ){

			u = new Searcher( this, new Point((int)(pt.getX()+(d.width)/2),(int)(pt.getY()+(d.height)/2)), p, r );
			current_agent = u;
			current_agent.setFloor(chart.getFloor().getNumber());
			users.add( current_agent);
			int index = users.indexOf(u);

		}else if( type.equalsIgnoreCase( "Rescuer" ) ){

			u = new Rescuer( this, new Point((int)(pt.getX()+(d.width)/2),(int)(pt.getY()+(d.height)/2)), p, r );
			current_agent = u;
			current_agent.setFloor(chart.getFloor().getNumber());
			users.add( current_agent);
			int index = users.indexOf(u);
		}


		updateAgentPosition( current_agent );

		try{
			fireControllerChange();
		}catch(Exception e){}


	} //method: addSearcher


    /* ===================================================================== */
    /** This method is for adding a new user into the simulator. The user is
     *  created based on the given information, and is added into the map, and
     *  all controller listeners are notified of the new addition.
     *
     *  @param  r   The range for the new user.
     *  @param  p   The port for the new user.
     *
     */
	public void addFire(int p, int r ){

		Point pt = frame.getChartPane().getScrollPosition();
		Dimension d= frame.getChartPane().getViewportSize();

		User u;

		u = new Fire( this, new Point((int)(pt.getX()+(d.width)/2),(int)(pt.getY()+(d.height)/2)), p, r );
		current_agent = u;
		current_agent.setFloor(chart.getFloor().getNumber());
		users.add( current_agent);
		int index = users.indexOf(u);

		updateAgentPosition( current_agent );

		try{
			fireControllerChange();
		}catch(Exception e){}


	} //method: addSearcher

    /* ===================================================================== */
    /** This method is for adding a new user into the simulator. The user is
     *  created based on the given information, and is added into the map, and
     *  all controller listeners are notified of the new addition.
     *
     *  @param  r   The range for the new user.
     *  @param  p   The port for the new user.
     *
     */
	public void addFire(int p, int r, Point pnt ){

		User u;

		u = new Fire( this, pnt, p, r );
		current_agent = u;
		current_agent.setFloor(chart.getFloor().getNumber());
		users.add( current_agent);
		int index = users.indexOf(u);

		updateAgentPosition( current_agent );

		try{
			fireControllerChange();
		}catch(Exception e){}


	} //method: addSearcher



    /* ===================================================================== */
    /** This method is for removing a user from the simulator. The actual
     *  user is given to the method, and the remove from the simulation, and
     *  all controller listeners are notified of the new removal.
     *
     *  @param  u   The User object to remove from the users list.
     *
     */
	public void removeUser(User u){

		users.remove(u);
		if( u instanceof Fire ){
			((Fire)u).killFire();
		}
		if( current_agent.getName().equals( u.getName()) && users.size() > 0){
			current_agent = (User)users.elementAt(0);
		}
		chart.repaint();
		fireControllerChange();


	} //method: removeUser





    /* ===================================================================== */
    /** This method is for adding a new access point into the simulator. The
     *  accesspoint is created based on the given information, and is added
     *  into the map, and all controller listeners are notified of the new
     *  addition.
     *
     *  @param  r   The range for the new Access Point
     *
     */
	public void addAccessPoint(int r ){

		Point pt = frame.getChartPane().getScrollPosition();
		Dimension d= frame.getChartPane().getViewportSize();

		AccessPoint u = new AccessPoint( " ", new Point((int)(pt.getX()+(d.width)/2),(int)(pt.getY()+(d.height)/2)), r );

		current_agent = u;
		points.add( u);

		u.setRange(r);
		u.setFloor(chart.getFloor().getNumber());
		u.setName( "access"+points.indexOf(u) );


		updateAgentPosition( u );

		try{
			fireControllerChange();
		}catch(Exception e){}


	} //method: addAccessPoint





    /* ===================================================================== */
    /** This method is for removing a access point from the simulator. The actual
     *  access point is given to the method, and the remove from the simulation, and
     *  all controller listeners are notified of the new removal.
     *
     *  @param  u   The Access Point object to remove from the points list.
     *
     */
	public void removeAccessPoint(AccessPoint u){

		points.remove(u);

		chart.repaint();
		fireControllerChange();

	} //method: removeAccessPoint





    /* ===================================================================== */
    /** This method updating an Agent's gps Position.  It will get the pixel
     *  location, and then turn this into a gps Position.
     *
     *  @param  u   The agent whose Position needs to be set by this method.
     *
     */
	public void updateAgentPosition(Agent u){

		// Get the location Point
		chart.updatePos(u.getLocation(), users, points);

		Point poi = u.getLocation();

		// Turn pixel into gps coordinates.
		Position chart_pos = chart.getChartData().transPos(poi);
		Position old = u.getPosition();

		//Retain some old Position information.
		chart_pos.setHeading( old.getHeading() );
		chart_pos.setTime( old.getTime() );


		u.setPosition( chart_pos );

	} //method: updateAgentPosition




    /* ===================================================================== */
    /** This method returns the Vector of all the current users in the simulator.
     *
     *  @return  Vector   All users in the simulation.
     *
     */
	public Vector getUsers(){

		return users;

	} //method: getUsers



    /* ===================================================================== */
    /** DANGEROUS METHOD TO USE!!!!!  Only use this when initialing the
     *  controller.  Any other time may crash iSIM.
     *
     *  @param  u   users to add to system.
     *
     */
	public void setUsers(Vector u){

		users = u;

	} //method: setUsers



    /* ===================================================================== */
    /** This method returns the Vector of all current access points in the
     *  simulator.
     *
     *  @return  Vector   All Access Points in the simulation.
     *
     */
	public Vector getPoints(){

		return points;

	} //method: getPoints



    /* ===================================================================== */
    /** DANGEROUS METHOD TO USE!!!!!  Only use this when initialing the
     *  controller.  Any other time may crash iSIM.
     *
     *  @param  u   access points to add to system.
     *
     */
	public void setPoints(Vector u){

		points = u;

	} //method: setUsers




    /* ===================================================================== */
    /** This method returns the current agent in focus or being moved around
     *  in the simulator map.
     *
     *  @return  Agent   The current Agent being moved or center on screen.
     *
     */
	public Agent getCurrentAgent(){

		return current_agent;

	} //method: getCurrentAgent




    /* ===================================================================== */
    /** This method sets who the current is.
     *
     *  @param  a   The Agent to make the current agent.
     *
     */
	public void setCurrentAgent(Agent a){

		current_agent = a;

	} //method: setCurrentAgent





    /* ===================================================================== */
    /** This method is for centering the given agent on the map.  Also returns
     *  the agent object in case it is needed once centered.
     *
     *  @param  name   The name of Agent to find and center on screen.
     *  @return  Agent   The Agent object that was centered.
     *
     */
	public Agent centerAgent(String name){

		Agent a = null;

		// Find the user or access point.
		for( int i=0; i <users.size(); i++ ){
			a = (Agent)users.elementAt(i);
			if( name.indexOf(""+a.getName()) > -1 ){
				current_agent = a;
				fireControllerChange();
				frame.centerScreen(a.getLocation());
			}
		}

		for( int i=0; i <points.size(); i++ ){
			a = (Agent)points.elementAt(i);
			if( name.indexOf(""+a.getName()) > -1 ){
				current_agent = a;
				fireControllerChange();
				frame.centerScreen(a.getLocation());
			}

		}

		return a;

	} //method: centerAgent





    /* ===================================================================== */
    /** This method is called when a User's location has changed.  A NMEA
     *  Message is created and then sent to the User's corresponding iPAQ via
     *  a port number.
     *
     *  @param  u   The user who has changed location.
     *
     */
	public void locationChange(User u){

		if( u.isLocationSensor() && u.isLocationSensor2() ){
			Position p = u.getPosition();
			Message msg = new NMEAMessage(u.getPort(), 2323, p.getLatitude(), p.getLongitude(), p.getHeading(), p.getTime());

			SocketSender ss = new SocketSender(msg);
			ss.start();
		}else if( !u.isLocationSensor() ){
			SocketSender s1 = new SocketSender( new ErrorMessage( u.getPort(),2323,new NMEAMessage(u.getPort(), 2323, 0, 0, 0, 0)));
			s1.start();
		}

	} //method: locationChange



    /* ===================================================================== */
    /** This method is called to send a message to all agents.
     *
     *  @param  msg   Message to send
     *
     */
	public void sendToAll(Sim2PMessage msg){

		User a = null;

		// Find the user or access point.
		for( int i=0; i <users.size(); i++ ){
			a = (User)users.elementAt(i);
			msg.setTo(a.getPort());

			SocketSender ss = new SocketSender(msg);
			ss.start();
		}


	} //method: sendToAll



    /* ===================================================================== */
    /** This method simply tests to see if the tray is visible, if not visible
     *  then it is assumed to be dead.
     *
     *  @return  boolean   True if tray is alive and visible, false otherwise.
     *
     */
	public boolean isTrayAlive(){

		try{
			if( tray.isVisible()){
				return true;
			}
		}catch(Exception e){
			return false;
		}

		return false;

	} //method: isTrayAlive




    /* ===================================================================== */
    /** This method creates a new TrayDialog for performing various functions
     *  in the simulator, and viewing information.
     *
     */
	public void makeTray(Frame f){
		try{
			tray.dispose();
		}catch(Exception e){}

		tray = new TrayDialog(this, f);
		fireControllerChange();

	} //method: makeTray




    /* ===================================================================== */
    /** This method returns the main JFrame for the iSIM  application.
     *
     *  @return  JFrame   The main JFrame for the simulation.
     *
     */
	public JFrame getFrame(){

		return frame;

	} //method: getFrame




    /* ===================================================================== */
    /** This method returns the ChartComp which holds the simulation map, and
     *  draws all the users.
     *
     *  @return  CharComp   The JComponent that is the map.
     *
     */
	public ChartComp getChartComp(){

    	return(chart);

  	} //method: getChartComp

    /* ===================================================================== */
    /** This method returns the ChartComp which holds the simulation map, and
     *  draws all the users.
     *
     *  @param  CharComp   The JComponent that is the map.
     *
     */
	public void setChartComp(ChartComp c){

    	chart = c;

  	} //method: getChartComp



    /* ===================================================================== */
    /** This method sets whether the wireless ranges should be displayed or
     *  not.
     *
     *  @param  ranges  True to show the wireless range on map, false otherwise.
     *
     */
	public void setRanges(boolean ranges){

		chart.setRanges(ranges);

	} //method: setRanges





    /* ===================================================================== */
    /** This method changes the current map/chart used in the simulation.
     *  The GUI Frame and ChartComp handle the details behind this.
     *
     *  @param  chartName   The new chart file name.
     *  @param  scale   The scale for the map image.
     *  @param  precision   The drawing position for lines on map.
     *
     */
  	public void changeChart(String chartName){


    	frame.changeChart(chartName);

  	} //method: changeChart




    /* ===================================================================== */
    /** This method is for reseting the data insdie the controller.  It removes
     *  all agents (users and access points).
     *
     */
	public void reset(){

		users = new Vector();
		points = new Vector();
		current_agent = null;
		chart.repaint();
		fireControllerChange();

	} //method: reset




    /* ===================================================================== */
    /** This method returns the pixel distance between two points.
     *
     *  @param  p1   The first point.
     *  @param  p2   The second point.
     *
     */
	public static double diff(Point p1, Point p2){

		try{
			double x = p1.getX() - p2.getX();
			if( x < 0 )
				x = x * -1;

			double y = p1.getY() - p2.getY();
			if( y < 0 )
				y = y * -1;

			if( x == 0 )
				return y;

			if( y == 0 )
				return x;

			return Math.sqrt( x*x + y*y );

		}catch(Exception e){
			System.out.println("Controller diff method: divide by zero "+e);
			e.printStackTrace();
			return 0.0;
		}


	} //method: diff



    /* ===================================================================== */
    /** This method is required by SocketReceiverListener and is called by the
     *  SocketReceiver object.  It delivers a message to us from some other
     *  client application wanting to send a message.
     *
     *  This method uses a multicasting approach and sends the message to all
     *  agents in range and up to a certain number of hops designated in the
     *  message itself.  It calls a Thread now to do the message passing, so
     *  that bandwidth delays can be simulated.
     *
     *  @param  msg   The incoming Message that the simulator has received.
     *
     */
	public void incomingMessage(Message msg){

		if( msg instanceof MulticastMessage || msg instanceof HelpMessage ){

			System.out.println("Received Multicast message from an iPAQ");

			Multicast ms = new Multicast( users, points, (P2PMessage)msg );
			ms.start();

		}else if( msg instanceof UnicastMessage ){

			System.out.println("Received Unicast message from an iPAQ");

			((UnicastMessage)msg).setTo( ((UnicastMessage)msg).getToPeer() );

			Unicast ms = new Unicast( users, points, (UnicastMessage)msg );
			ms.start();

		}else if( msg instanceof RangeMessage ){

			System.out.println("Received range message from an iPAQ");

			RangeMessage rng = new RangeMessage(msg.getFrom(), msg.getTo(), ((RangeMessage)msg).getAgent(), ((RangeMessage)msg).getData() );
			rng.setInRange( isInRange(((RangeMessage)msg)) );

			SocketSender ss = new SocketSender(rng);
			ss.start();

		} //if


	} //method: incomingMessage


	public boolean isInRange( RangeMessage msg ){

		int from = msg.getFrom();
		int to = msg.getAgent();

		double diff = 0;
		Agent a = null;
		Agent b = null;
		Agent temp = null;
		Vector routers = new Vector();

		// Find the agent that sent the message.
		for( int i=0; i <users.size(); i++ ){
			temp = (Agent)users.elementAt(i);
			if( from == ((User)temp).getPort() ){
				a = temp;
			}
		}

		// Find the agent that the message will go to.
		for( int i=0; i <users.size(); i++ ){
			temp = (Agent)users.elementAt(i);
			if( to == ((User)temp).getPort() ){
				b = temp;
			}
		}

		routers.add(a);

		try{

		//Now send message to only one agent if in range.
		if( a != null && b != null){

			// Send to all agents in one hop.
			for( int i=0; i <users.size(); i++ ){

				temp = (Agent)users.elementAt(i);
				diff = Controller.diff(a.getLocation(),temp.getLocation());

				//System.out.println( "diff: "+diff);

				if( (diff < (a.getRange()+temp.getRange())) && !(routers.contains(temp)) ){;

					routers.add(temp);
				}
			}

			for( int i=0; i <points.size(); i++ ){
				temp = (Agent)points.elementAt(i);
				diff = Controller.diff(a.getLocation(),temp.getLocation());

				if( diff < (a.getRange()+temp.getRange()) && !(routers.contains(temp)) ){
					routers.add(temp);
				}
			}


			// Now find those within hops.
			int h = 1;
			while( h < 10 ){
				h++;

				for( int j=0; j < routers.size(); j++ ){

					a = (Agent)routers.elementAt(j);

					// Find all agents in one hop.
					for( int i=0; i <users.size(); i++ ){

						temp = (Agent)users.elementAt(i);
						diff = Controller.diff(a.getLocation(),temp.getLocation());

						if( !(routers.contains(temp)) && diff < (a.getRange()+temp.getRange()) ){

							routers.add(temp);
						}
					}
					for( int i=0; i <points.size(); i++ ){

						temp = (Agent)points.elementAt(i);
						diff = Controller.diff(a.getLocation(),temp.getLocation());

						if( !(routers.contains(temp)) && diff < (a.getRange()+temp.getRange()) ){
							routers.add(temp);
						}
					}

				}
			} //while


			//If the agent can be reached then send the message
			if( routers.contains(b) ){

				return true;
			}else{
				return false;
			}


		}

		}catch(Exception e){System.out.println("Error in finding range "+e);}
		return false;

	}

	/* ===================================================================== */
    /** This methods adds a listener to the list of listeners this controller
     *  knows about.
     *
     *  @param  listener
     *
     */
    public void addControllerListener(ControllerListener listener){

		listeners.add(listener);

    } //method: addControllerListener




	/* ===================================================================== */
    /** This method removes a listener from the list of listeners that
     *  this controller knows about.
     *
     *  @param  listener
     *
     */
    public void removeControllerListener(ControllerListener listener){

		listeners.remove(listener);

    } //method: removeControllerListener




	/* ===================================================================== */
    /** This method notifies all subscribed listeners that they must
     *  refresh themselves.
     */
    public void fireControllerChange(){

		ControllerListener[] informMe = new ControllerListener[listeners.size()];
		informMe = (ControllerListener[])listeners.toArray(informMe);

		for(int i = 0; i < informMe.length; i++){

			informMe[i].refresh();
		}

    } //method: fireControllerChange



} //class: Controller