/**<pre>
 * Title: GPS Simulator
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package gui;

import agent.*;
import agent.autonomous.*;
import controller.*;

import java.awt.image.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;



/* ====================================================================== */
/** This is the JDialog is designed to be a simple display Window for
 *  display a User's (not access point) information.  Some of the information
 *  can be edited at this point.  Also a photo, of the user's real world
 *  location is displayed if exists.
 *
 */
public class PropDialog extends JDialog implements ControllerListener, DocumentListener{

	/** Layout used in this panel. */
    private BorderLayout bLayMain = new BorderLayout();

    /** Layout used in this panel. */
    private GridBagLayout gbagLayInfo = new GridBagLayout();

	/** Main panel to hold all components */
    private JPanel pnlMain = new JPanel();

    /** The parent Frame that owns this JDialog. */
    private Frame parentFrame;

	/** The controller of the simulator. */
    private Controller controller;

	/** An error notification if user types letters in the duration column. */
	private JDialog errorDialog;

	/** Lots of fields for User's stuff. */
	private JTextField nameField = new JTextField(14);
	private JTextField portField = new JTextField(14);
	private JTextField rangeField = new JTextField(14);
	private JTextField headingField = new JTextField(14);
	private JTextField latField = new JTextField(14);
	private JTextField lngField = new JTextField(14);

	private JTextField floorField = new JTextField(14);
	private JTextField healthField = new JTextField(14);
	private JTextField oxygenField = new JTextField(14);
	private JTextField weightField = new JTextField(14);

	/** A button to close this dialog. */
	private JButton btnClose = new JButton("Close");

	/** The image of the user's location if exists. */
	private ImageIcon icoLogo = new ImageIcon("files/photos/nopic.gif");

	/** A label for the user's location photo. */
	private JLabel lblPicture = new JLabel(icoLogo);


	private SensorPanel sensors;

    /* ===================================================================== */
    /** Constructor that creates this JDialog. The parent and controller must
     *  not be null for this Dialog to function properly.
     *
     *  @param  parent   The owning frame of this dialog.
     *  @param  controller   The simulation controller.
     *
     */
    public PropDialog(Controller controller, Frame parent ) {

		super(parent);
        setTitle("User Properties");

        this.parentFrame = parent;
        this.controller = controller;
        controller.addControllerListener(this);

		// Setup the gui
        try {
          init();
        }
        catch(Exception e) {
          e.printStackTrace();
        }


		// Set the size and location

        this.setSize(620, 400);
        this.pack();

		//Center this dialog on the screen.
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		int x = ( screen.width / 2) - (this.getSize().width /2 );
		int y = ( screen.height / 2) - (this.getSize().height /2 );
        this.setLocation(x,y);

		// Show us.
        this.setVisible(true);

    } //Construtor




    // =======================================================================
    /** Initializes the GUI with components.
     *
     */
    private void init() throws Exception {

		// Setup layouts and borders
        bLayMain.setHgap(5);
        bLayMain.setVgap(5);
        pnlMain.setBorder(TTBorder.getBorder("User Properties"));
        this.getContentPane().setLayout(bLayMain);
        pnlMain.setLayout(new BorderLayout());

		JPanel btnPanel = new JPanel();
        btnPanel.setLayout(gbagLayInfo);

        btnPanel.add(btnClose, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.NONE,
                    new Insets(5, 5, 5, 5), 0, 0));

		// Setup the action listeners
		btnClose.setMnemonic(KeyEvent.VK_C);
        btnClose.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(ActionEvent e) {
                  btdClose_actionPerformed();
              }
        });



		JLabel lblName = new JLabel("User Name:");
		JLabel lblPort = new JLabel("Port:");
		JLabel lblRange = new JLabel(" Wireless Range:");
		JLabel lblHeading = new JLabel("GPS Heading:");
		JLabel lblLat = new JLabel("GPS Latitude:");
		JLabel lblLng = new JLabel("GPS Longitude:");

		JLabel lblFloor = new JLabel("On floor number:");
		JLabel lblOxygen = new JLabel("Oxygen level:");
		JLabel lblHealth = new JLabel("Health level:");
		JLabel lblWeight = new JLabel("Carry weight:");


		nameField.setEditable(true);
		headingField.setEditable(false);
		latField.setEditable(false);
		lngField.setEditable(false);

		// Add stuff to panels
		JPanel stats = new JPanel();
		stats.setLayout(gbagLayInfo);

		sensors = new SensorPanel(controller, this);


        pnlMain.add(stats, BorderLayout.EAST);
        pnlMain.add(lblPicture, BorderLayout.WEST);
        this.getContentPane().add(pnlMain, BorderLayout.WEST);
        this.getContentPane().add(btnPanel, BorderLayout.SOUTH);
        this.getContentPane().add(sensors, BorderLayout.CENTER);


        stats.add(lblName,    new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(lblPort,     new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(lblRange,    new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
                     GridBagConstraints.WEST,
                     GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(lblHeading,   new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(lblLat,   new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(lblLng,   new GridBagConstraints(0, 5, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(lblFloor,   new GridBagConstraints(0, 6, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));
        stats.add(lblHealth,   new GridBagConstraints(0, 7, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));
        stats.add(lblOxygen,   new GridBagConstraints(0, 8, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));
        stats.add(lblWeight,   new GridBagConstraints(0, 9, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));



        stats.add(nameField,    new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(portField,     new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(rangeField,    new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0,
                     GridBagConstraints.WEST,
                     GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(headingField,   new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(latField,   new GridBagConstraints(1, 4, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(lngField,   new GridBagConstraints(1, 5, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(floorField,   new GridBagConstraints(1, 6, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));
        stats.add(healthField,   new GridBagConstraints(1, 7, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));
        stats.add(oxygenField,   new GridBagConstraints(1, 8, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));
        stats.add(weightField,   new GridBagConstraints(1, 9, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));


		// Make our size fixed.
        this.setResizable(false);



    } //method: init



    /* ===================================================================== */
    /** This method is from being a ControllerListener, everytime the Controller
     *  notifies us, we refresh the user information in the all fields, and check
     *  for a new photo of the current location and heading.
     *
     */
	public void refresh(){

		Agent a = controller.getCurrentAgent();

		if( a instanceof Fire){

			portField.getDocument().removeDocumentListener(this);
			nameField.getDocument().removeDocumentListener(this);
			rangeField.getDocument().removeDocumentListener(this);
			floorField.getDocument().removeDocumentListener(this);
			healthField.getDocument().removeDocumentListener(this);
			oxygenField.getDocument().removeDocumentListener(this);
			weightField.getDocument().removeDocumentListener(this);

			nameField.setText(((User)a).getUsername());
			portField.setText(""+((User)a).getPort());
			rangeField.setText(""+a.getRange());
			headingField.setText("n/a");
			latField.setText(""+a.getPosition().getLatitude());
			lngField.setText(""+a.getPosition().getLongitude());
			floorField.setText(""+a.getFloor());
			healthField.setText(""+a.getHealth());
			oxygenField.setText("n/a");
			weightField.setText("n/a");

			portField.getDocument().addDocumentListener(this);
			nameField.getDocument().addDocumentListener(this);
			rangeField.getDocument().addDocumentListener(this);
			floorField.getDocument().addDocumentListener(this);
			healthField.getDocument().addDocumentListener(this);
			oxygenField.getDocument().addDocumentListener(this);
			weightField.getDocument().addDocumentListener(this);

			// Try to load a photo here based on heading and position!!!
			// pnlMain.remove(lblPicture);
			// Multiply lat and lng by 100 to get precise location.
			int lat = (int)(100 * a.getPosition().getLatitude());
			int lng = (int)(100 * a.getPosition().getLongitude());
			int hea = (int)(a.getPosition().getHeading());

			try{

				icoLogo = new ImageIcon("files/photos/photo"+lat+"_"+lng+"_"+hea+".jpg");
				if( icoLogo.getIconHeight() < 10 ){

					icoLogo = new ImageIcon("files/photos/nopic.gif");
				}

				lblPicture.setIcon(icoLogo);


			}catch(Exception exc){

				icoLogo = new ImageIcon("files/photos/nopic.gif");
				lblPicture.setIcon(icoLogo);
			}


		}else if( a instanceof User){

			portField.getDocument().removeDocumentListener(this);
			nameField.getDocument().removeDocumentListener(this);
			rangeField.getDocument().removeDocumentListener(this);
			floorField.getDocument().removeDocumentListener(this);
			healthField.getDocument().removeDocumentListener(this);
			oxygenField.getDocument().removeDocumentListener(this);
			weightField.getDocument().removeDocumentListener(this);
			sensors.location.removeChangeListener(sensors);
			sensors.bio.removeChangeListener(sensors);
			sensors.room.removeChangeListener(sensors);
			sensors.wireless.removeChangeListener(sensors);
			sensors.location2.removeChangeListener(sensors);
			sensors.bio2.removeChangeListener(sensors);
			sensors.room2.removeChangeListener(sensors);
			sensors.wireless2.removeChangeListener(sensors);



			nameField.setText(((User)a).getUsername());
			portField.setText(""+((User)a).getPort());
			rangeField.setText(""+a.getRange());
			headingField.setText(""+a.getPosition().getHeading());
			latField.setText(""+a.getPosition().getLatitude());
			lngField.setText(""+a.getPosition().getLongitude());
			floorField.setText(""+a.getFloor());
			healthField.setText(""+a.getHealth());
			oxygenField.setText(""+a.getOxygen());
			weightField.setText(""+a.getWeight());
			sensors.bio.setSelected(!a.isBioSensor());
			sensors.location.setSelected(!a.isLocationSensor());
			sensors.room.setSelected(!a.isRoomSensor());
			sensors.wireless.setSelected(!a.isWirelessSensor());
			sensors.bio2.setSelected(!a.isBioSensor2());
			sensors.location2.setSelected(!a.isLocationSensor2());
			sensors.room2.setSelected(!a.isRoomSensor2());
			sensors.wireless2.setSelected(!a.isWirelessSensor2());


			portField.getDocument().addDocumentListener(this);
			nameField.getDocument().addDocumentListener(this);
			rangeField.getDocument().addDocumentListener(this);
			floorField.getDocument().addDocumentListener(this);
			healthField.getDocument().addDocumentListener(this);
			oxygenField.getDocument().addDocumentListener(this);
			weightField.getDocument().addDocumentListener(this);
			sensors.location.addChangeListener(sensors);
			sensors.bio.addChangeListener(sensors);
			sensors.room.addChangeListener(sensors);
			sensors.wireless.addChangeListener(sensors);
			sensors.location2.addChangeListener(sensors);
			sensors.bio2.addChangeListener(sensors);
			sensors.room2.addChangeListener(sensors);
			sensors.wireless2.addChangeListener(sensors);

			// Try to load a photo here based on heading and position!!!
			// pnlMain.remove(lblPicture);
			// Multiply lat and lng by 100 to get precise location.
			int lat = (int)(100 * a.getPosition().getLatitude());
			int lng = (int)(100 * a.getPosition().getLongitude());
			int hea = (int)(a.getPosition().getHeading());

			try{

				icoLogo = new ImageIcon("files/photos/photo"+lat+"_"+lng+"_"+hea+".jpg");
				if( icoLogo.getIconHeight() < 10 ){

					icoLogo = new ImageIcon("files/photos/nopic.gif");
				}

				lblPicture.setIcon(icoLogo);


			}catch(Exception exc){

				icoLogo = new ImageIcon("files/photos/nopic.gif");
				lblPicture.setIcon(icoLogo);
			}

		} //if

	} //method: refresh



   	/* ====================================================================== */
	/** This method is called whenever user edits a value in JTextField.
	 *
	 *  @param  e  The document event from one of our textfields.
	 *
	 */
    public void insertUpdate(DocumentEvent e) {

		//Try to grab text from textfields and save it to the current cd.
   	 	try{

			//set these values.  year must be an int or else it will throw
			// exception.
			if( controller.getCurrentAgent() instanceof Fire){

				controller.getCurrentAgent().setRange( Integer.parseInt(rangeField.getText()) );
				controller.getCurrentAgent().setFloor( Integer.parseInt(floorField.getText()) );
				controller.getCurrentAgent().setHealth( Integer.parseInt(healthField.getText()) );
				((User)controller.getCurrentAgent()).setPort( Integer.parseInt(portField.getText()) );

			}else if( controller.getCurrentAgent() instanceof User){

				controller.getCurrentAgent().setRange( Integer.parseInt(rangeField.getText()) );
				controller.getCurrentAgent().setFloor( Integer.parseInt(floorField.getText()) );
				controller.getCurrentAgent().setHealth( Integer.parseInt(healthField.getText()) );
				controller.getCurrentAgent().setOxygen( Integer.parseInt(oxygenField.getText()) );
				controller.getCurrentAgent().setWeight( Integer.parseInt(weightField.getText()) );
				((User)controller.getCurrentAgent()).setPort( Integer.parseInt(portField.getText()) );
				((User)controller.getCurrentAgent()).setUsername( nameField.getText() );

			}

			controller.removeControllerListener(this);
			controller.fireControllerChange();
			controller.addControllerListener(this);

		//This exception is thrown if the uses types a non-numeric character
		// into the duration column.
		}catch(NumberFormatException nfe){

			//This is outdated. Just use java's predefined dialog for this.

			//Setup a current/new JDialog.
			errorDialog = new JDialog(this, "Error");

			//Create a button and the labels.
			JLabel lbl = new JLabel("Please only enter numbers in the fields.");
			JLabel lblPic = new JLabel( new ImageIcon("files/photos/info.jpg") );
			JButton btn = new JButton( "OK");

			//Create panels for hold the Components
			JPanel btnpanel = new JPanel();
			JPanel lblpanel = new JPanel();
			lblpanel.setBorder( BorderFactory.createEmptyBorder(15, 15, 5, 15) );

			//Add components to the panels, then add panels to the JDialog.
			lblpanel.add(lblPic, BorderLayout.WEST);
			lblpanel.add(lbl, BorderLayout.EAST);
			btnpanel.add(btn);

			errorDialog.getContentPane().add(lblpanel, BorderLayout.CENTER);
			errorDialog.getContentPane().add(btnpanel, BorderLayout.SOUTH);

			//Make a button listener to close the dialog.
			btn.setMnemonic(KeyEvent.VK_O);
			btn.addActionListener(new ActionListener() {
            	public void actionPerformed(ActionEvent e) {
            	    errorDialog.dispose();
            	}
        	});

			//Pack it all together
        	errorDialog.pack();

			//Center this dialog on the screen.
			Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
			int x = ( screen.width / 2) - (errorDialog.getSize().width /2 );
			int y = ( screen.height / 2) - (errorDialog.getSize().height /2 );
			errorDialog.setLocation( x, y);

			//Now show this dialog.
        	errorDialog.setVisible(true);


			//Now make sure dialog is in front and in focus.
        	errorDialog.toFront();


		} //try

    } //method: insertUpdate


   	/* ====================================================================== */
	/** Don't care about this method.  Only need it to satisfy DocumentListener
	 *  interface.
	 *
	 *  @param  e  The event we don't care about.
	 *
	 */
    public void removeUpdate(DocumentEvent e){}
    /* ====================================================================== */
 	/** Don't care about this method.  Only need it to satisfy DocumentListener
 	 *  interface.
 	 *
 	 *  @param  e  The event we don't care about.
 	 *
	 */
 	public void changedUpdate(DocumentEvent e){}
    /* ====================================================================== */
 	/** Don't care about this method.  Only need it to satisfy DocumentListener
 	 *  interface.
 	 *
 	 *  @param  e  The event we don't care about.
 	 *
	 */
 	private void calculateValue(DocumentEvent e){}


	public void btdClose_actionPerformed(){

		this.dispose();

	}


} //class: PropDialog
