/**<pre>
 * Title: GPS Simulator
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package gui.tray;

import recorder.*;
import gui.*;
import agent.*;
import agent.autonomous.*;
import util.*;
import controller.*;

import java.awt.*;
import javax.swing.border.*;
import javax.swing.table.*;
import javax.swing.*;
import java.util.*;
import java.awt.event.*;



/* ====================================================================== */
/** This class is the People tab in the TrayDialog.  It displays a list
 *  of all users in the simulation, and enables adding and removing of users.
 *
 */
public class PnlTabRecorder extends JPanel implements ControllerListener{

	/** The simulation controller that keeps all data. */
	private Controller controller;

	/** Layout used in this panel. */
    private BorderLayout bLayMain = new BorderLayout();

    /** Layout used in this panel. */
    private GridBagLayout gbagLayInfo = new GridBagLayout();

    //Components in this JPanel

    /** Panel for buttons */
    private JPanel btnPanel = new JPanel();

    /** Add user button. */
    private JButton btnAdd = new JButton("Record");

    /** REmove user button. */
    private JButton btnRemove = new JButton(" Play ");

     private JButton btnAll = new JButton("Play All");

    /** Scrollpane for JTable */
    private JScrollPane spnUsers = new JScrollPane();

    /** JTable for displaying users */
    private JTable tableUsers = new JTable();

    /** The JTable's columns. */
    private String[] colNames = { "User", "Port", "Latitude", "Longitude"};

	/** A popup dialog for adding new user. */
	private JDialog addDialog;

    /** Field for user port number */
    private JTextField port_field;

    /** Field for user port number */
    private JTextField name_field;

    /** Field for uer range. */
    private JTextField range_field;


	private RecorderThread recorder;



    // ======================================================================
    /** Constructor sets up this Panel for a tab and calls init.
     *  It also signs us up as a controller listener.
     *
     */
    public PnlTabRecorder( Controller controller ) {

		this.controller = controller;
		controller.addControllerListener(this);

		recorder = new RecorderThread(controller);

        try {
            init();
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }

    } //Constructor




    // ======================================================================
    /** Initializes this JPanel with Components.  Adds the table, buttons,
     *  and sets the action listeners.
     *
     */
    private void init() throws Exception {

		// Set the borders and layout
        bLayMain.setHgap(5);
        bLayMain.setVgap(5);
        this.setBorder(TTBorder.getBorder("People"));
        this.setLayout(bLayMain);
        btnPanel.setLayout(gbagLayInfo);

		//  Add stuff to the panels and table
        this.add(btnPanel, BorderLayout.SOUTH);

        btnPanel.add(btnAdd, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.NONE,
                    new Insets(5, 10, 0, 0), 0, 0));

        btnPanel.add(btnRemove, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(5, 10, 0, 0), 0, 0));

        btnPanel.add(btnAll, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(5, 10, 0, 0), 0, 0));

		// Set up the table and scrollpane
        this.add(spnUsers,  BorderLayout.CENTER);

		String[][] rowData = new String[0][colNames.length]; // The data array


        DefaultTableModel dtm = new DefaultTableModel(rowData, colNames) {
            // set this to non-editable table model.
            public boolean isCellEditable(int row, int col) {
                return false;
            }
        };
        tableUsers = new JTable(dtm);

        spnUsers.getViewport().add(tableUsers, 0);


		// Setup the action listeners
		btnAdd.setMnemonic(KeyEvent.VK_R);
        btnAdd.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(ActionEvent e) {
                  btdRecord_actionPerformed();
              }
        });
		btnAll.setMnemonic(KeyEvent.VK_A);
        btnAll.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(ActionEvent e) {
                  btdPlayAll_actionPerformed();
              }
        });

		btnRemove.setMnemonic(KeyEvent.VK_P);
        btnRemove.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                btdPlay_actionPerformed();
            }
        });

		// Set panel size (gets over ridden by TrayDialog's set size)
        this.setSize(new Dimension(550, 300));

    } //method: init




    // ========================================================================
    /** This method is called to add a new user to the simulation.
     *
     */
    public void btdRecord_actionPerformed() {

		TableModel tm = tableUsers.getModel();

		if( (btnAdd.getText()).equals("Record") ){

			int row = tableUsers.getSelectedRow();
			if( row > -1 ){

				btnAdd.setText("Stop");

				String name = (String)tm.getValueAt(row, 0);

				Vector users = controller.getUsers();
				User use;

				for (int i=0; i < users.size(); i++) {
					use = (User)users.get(i);
					if(use.getName().equals(name) ){
						recorder.startRecording(use);
					}
				}
			}else{
			JOptionPane.showMessageDialog(addDialog,
				"Please select an agent to record.",
				"Cannot record",
				JOptionPane.INFORMATION_MESSAGE );
			this.requestFocus();
			return;
			}
		}else{
			recorder.stopRecording();
			btnAdd.setText("Record");
		}



    } //method:




    // ========================================================================
    /** This method is called to remove a user from the simulation.
     *
     */
    public void btdPlay_actionPerformed() {

		TableModel tm = tableUsers.getModel();

		int row = tableUsers.getSelectedRow();
		if( row > -1 ){

			String name = (String)tm.getValueAt(row, 0);

			Vector users = controller.getUsers();
			User use;

			for (int i=0; i < users.size(); i++) {
				use = (User)users.get(i);
				if(use.getName().equals(name) && use.getRecording() != null ){

					//play the agent's record here.
					System.out.println("Lets play the agent");
					new PlayerThread(use, controller);

				}
			}
		}else{
			JOptionPane.showMessageDialog(addDialog,
				"Please select an agent to play.",
				"Cannot play",
				JOptionPane.INFORMATION_MESSAGE );
			this.requestFocus();
			return;
		}

    } //method: btdRemove_actionPerformed


    // ========================================================================
    /** This method is called to remove a user from the simulation.
     *
     */
    public void btdPlayAll_actionPerformed() {

		Vector users = controller.getUsers();
		User use;

		for (int i=0; i < users.size(); i++) {
			use = (User)users.get(i);
			if(use.getRecording() != null ){

				//play the agent's record here.
				System.out.println("Lets play the agents");
				new PlayerThread(use, controller);

			}
		}

    } //method: btdRemove_actionPerformed







    // ========================================================================
    /** This method required by ControllerListener interface.
     *  This method resets the data in the JTable, so as to keep it up to date.
     *
     */
	public void refresh(){

		Vector users = controller.getUsers();
		Position pos;
		User use;


		int lengthy = 0;
		for (int i=0; i < users.size(); i++) {
			use = (User)users.get(i);
			if( !(use instanceof Searcher || use instanceof Rescuer || use instanceof Fire) ){
				lengthy++;
			}
		}

		String[][] rowData = new String[lengthy][colNames.length]; // The data array

		int k = 0;
		for (int i=0; i < users.size(); i++) {
			use = (User)users.get(i);
			pos = use.getPosition();

			if( !(use instanceof Searcher || use instanceof Rescuer || use instanceof Fire)){
	  			rowData[k][0] = use.getUsername();
	  			rowData[k][1] = ""+use.getPort();
	  			rowData[k][2] = ""+pos.getLatitude();
	  			rowData[k][3] = ""+pos.getLongitude();
	  			//rowData[k][4] = "";
				k++;
			}

		}


        DefaultTableModel dtm = new DefaultTableModel(rowData, colNames) {
            // set this to non-editable table model.
            public boolean isCellEditable(int row, int col) {
                return false;
            }
        };
        tableUsers = new JTable(dtm);

        spnUsers.getViewport().add(tableUsers, 0);


	} //method: refresh


} //class: PnlTabPeople