/**<pre>
 * Title: iSIM
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package map;

import util.*;

import java.util.*;


/* ====================================================================== */
/** This class
 *
 *  1 foot = .000003 degrees of latitude/longitude
 */
public class Wall implements java.io.Serializable{


	/** The gps Position of 1st endpoint. */
  	private Position position1;

	/** The gps Position of 2nd endpoint. */
  	private Position position2;

	/** The wireless signal degredation caused by this wall. */
  	private double rf_degredation;

	/** The sound level degredation caused by this wall. */
  	private double sound_degredation;

	/** The list of openings in this wall object. */
  	private Vector openings = new Vector();

  	/** The height of the wall. */
  	private int height;

	/** Filename of image texture. */
	private String texture_filename;

    /* ===================================================================== */
    /** This constructor initializes the
     *
     *  @param  p1   The first endpoint.
     *  @param  p2   The second endpoint.
     *  @param  rf   The rf degredation.
     *  @param  sd   The sound degredation.
     *  @param  h    The height.
     *  @param  f    Thewall texture.
     *
     */
  	public Wall(Position p1, Position p2, double rf, double sd, int h, String f){

		position1 = p1;
		position2 = p2;
		rf_degredation = rf;
		sound_degredation = sd;
		height = h;
		texture_filename = f;

	} //Constructor




    /* ===================================================================== */
    /** This method returns the
     *
     *  @return
     *
     */
	public Vector getOpenings(){

		return openings;

	} //method:




    /* ===================================================================== */
    /** This method sets the
     *
     *  @param
     *
     */
	public void addOpening(Opening o){

		openings.addElement(o);

	} //method:



    /* ===================================================================== */
    /** This method returns the
     *
     *  @return
     *
     */
	public Position getPosition1(){

		return position1;

	} //method:

    /* ===================================================================== */
    /** This method returns the
     *
     *  @return
     *
     */
	public Position getPosition2(){

		return position2;

	} //method:

    /* ===================================================================== */
    /** This method returns the
     *
     *  @param
     *
     */
	public void setPosition1(Position p){

		position1 = p;

	} //method:

    /* ===================================================================== */
    /** This method returns the
     *
     *  @param
     *
     */
	public void setPosition2(Position p){

		position2 = p;

	} //method:


    /* ===================================================================== */
    /** This method returns the
     *
     *  @return
     *
     */
	public double getRFDegredation(){

		return rf_degredation;

	} //method:

    /* ===================================================================== */
    /** This method returns the
     *
     *  @return
     *
     */
	public double getSoundDegredation(){

		return sound_degredation;

	} //method:


    /* ===================================================================== */
    /** This method returns the
     *
     *  @return
     *
     */
	public void setRFDegredation(double d){

		rf_degredation = d;

	} //method:

    /* ===================================================================== */
    /** This method returns the
     *
     *  @return
     *
     */
	public void setSoundDegredation(double d){

		sound_degredation = d;

	} //method:



    /* ===================================================================== */
    /** This method returns the
     *
     *  @return
     *
     */
	public int getHeight(){

		return height;

	} //method:

    /* ===================================================================== */
    /** This method returns the
     *
     *  @return
     *
     */
	public void setHeight(int d){

		height = d;

	} //method:


    /* ===================================================================== */
    /** This method returns the
     *
     *  @return
     *
     */
	public void setTexture(String d){

		texture_filename = d;

	} //method:

    /* ===================================================================== */
    /** This method returns the
     *
     *  @return
     *
     */
	public String getTexture(){

		return texture_filename;

	} //method:


} //class: Wall

