/**<pre>
 * Title: iSIM
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package transport.protocol;

import java.io.*;
import java.awt.*;


/* ====================================================================== */
/** This class represents a Multicast message.  This message is sent by
 *  agents to the simulator, the simulator then sends the message to all
 *  agents that are with within range and certain number of hops.
 *
 */
public class HelpMessage extends MulticastMessage implements Serializable{

	/** The string content of this Message. */
  	private String message;


	private Point location;


    /* ===================================================================== */
    /** This constructor initializes the Message.  Object content must be added
     *  seperately with setContent() method, and the object must be
     *  Serializable.
     *
     *  @param  message   The String message.
     *  @param  from   The port the message sender listens to.
     *
     */
  	public HelpMessage(String message, int from, Point loc){
		super(message, from);

		this.message = message;
		this.location = loc;

	} //Constructor



    /* ===================================================================== */
    /** This constructor initializes the Message.  Object content must be added
     *  seperately with setContent() method, and the object must be
     *  Serializable.
     *
     *  @param  message   The String message.
     *  @param  from   The port the message sender listens to.
     *  @param  hops   The max number of hops.
     *
     */
  	public HelpMessage(String message, int from, int hops, Point loc){
		super(message, from, hops);

		this.message = message;
		this.location = loc;

	} //Constructor


    /* ===================================================================== */
    /** This method returns the String content of the message.
     *
     *  @return  String   The message.
     *
     */
	public String getMessage(){

		return message;

	} //method: getMessage



    /* ===================================================================== */
    /** This method returns the String content of the message.
     *
     *  @return  String   The message.
     *
     */
	public Point getLocation(){

		return location;

	} //method: getLocation



} //class: MulticastMessage

